package Net::Paraget::Server;
#
# $Id: Server.pm,v 1.1.1.1 2001/01/31 19:40:39 ftobin Exp $
#

use strict;

use Class::MethodMaker
  get_set       => [ qw( host scheme overhead list amount_completed work_time ) ],
  boolean       => [ qw( disabled ) ],
  list          => [ qw( mirrors ) ],
  new_hash_init => 'new';


sub speed
{
    my ( $self, $bad_arg ) = @_;
    die "You can't set speed" if defined $bad_arg;
    my $distance = $self->amount_completed();
    my $time     = $self->work_time();
    return undef unless $time;
    
    return int( $distance / $time );
}


sub as_string
{
    my ( $self ) = @_;
    return lc join ( '', $self->scheme(), '://', $self->host() );
}


sub get_mirror
{
    my ( $self ) = @_;
    return $self->mirrors->[0];
}


sub time_over_interval
{
    my ( $self, $interval ) = @_;
    
    my $prev = $interval->prev();
    
    my $prev_speed = 0;
    my $prev_assignment = $prev->assignment();
    $prev_speed = $prev_assignment->server->a_speed() if $prev_assignment;
    
    my $distance = $interval->size();
    my $overhead = $self->an_overhead();
    my $total_speed = $self->a_speed() + $prev_speed;
    
    my $time = $overhead + $distance / $total_speed;
    
    return $time;
}


sub get_a
{
    my ( $self, $value ) = @_;
    my $average_string = "average_$value";
    my $default_string = "default_$value";
    
    my $a = ( $self->$value()
	      || $self->list->$average_string()
	      || $self->list->$default_string()
	    );
    return $a;
}

sub a_speed
{
    my ( $self ) = @_;
    return $self->get_a( 'speed' );
}


sub an_overhead
{
    my ( $self ) = @_;
    return $self->get_a( 'overhead' );
}


sub id
{
    my ( $self ) = @_;
    $self->as_string();
}


sub info
{
    my ( $self ) = @_;
    return $self->id();
}


sub update_from_report
{
    my ( $self, $report ) = @_;
    
    foreach my $field ( qw( amount_completed work_time ) )
    {
	my $prev = $self->$field() || 0;
	$self->$field( $prev + $report->$field() );
    }
    
    $self->decrease_speed_history( 50 );
    
    my $speed = $self->speed();
    $speed = 'undefined' if not defined $speed;
}


sub count_able_mirrors
{
    my ( $self ) = @_;
    
    my $count = 0;
    foreach my $m ( $self->mirrors() )
    {
	
	$count++ if not $m->disabled();
    }
    return $count;
}


sub decrease_speed_history
{
    my ( $self, $decrease_to_percent ) = @_;
    
    my $original_completed = $self->amount_completed();
    my $original_time      = $self->work_time();
    
    my $multiplier = $decrease_to_percent / 100;
    
    my $new_completed = int( $original_completed * $multiplier );
    my $new_time      = int( $original_time * $multiplier );
    
    $self->amount_completed( $new_completed );
    $self->work_time( $new_time );
}


1;
